package com.allinpay.demo.util;

import java.security.Provider;
import java.security.SecureRandom;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * @Description
 * @Author meixf@allinpay.com
 * @Date 2018年7月8日
 **/
public class AIPGCipher {
	
	private static Log log = LogFactory.getLog(AIPGCipher.class);
	
	private Provider prv = null;
	private String encoding = "GBK";
	
	public AIPGCipher(){
		
	}
	
	public AIPGCipher(String encoding){
		this.encoding = encoding;
	}
	
	public AIPGCipher(Provider provide){
		this.prv = provide;
	}
	
	public AIPGCipher(String encoding, Provider provide){
		this.prv = provide;
		this.encoding = encoding;
	}
	
	/**
	 * @param mw 明文
	 * @return
	 */
	public String encrypt(String mw){
		try{
			byte[] bs = mw.getBytes(encoding);
			byte[] ks = "12345678".getBytes(encoding);
			return byte2hex(encrypt(bs, ks));
		}catch (Exception e) {
			log.error("DES算法，加密出错 ，" + e.getMessage());
			throw new RuntimeException("DES算法，加密数据出错!", e);
		}
	}
	
	/**
	 * @param mw 密文
	 * @return
	 */
	public String decrypt(String mw){
		try{
			byte[] bs = hex2byte(mw.getBytes(encoding));
			byte[] ks = "12345678".getBytes(encoding);
			return new String(decrypt(bs, ks), encoding);
		}catch (Exception e) {
			throw new RuntimeException("DES算法，解密数据出错!", e);
		}
	}
	/**
	 * 加密函数
	 * @param data 加密数据
	 * @param key 密钥
	 * @return 返回加密后的数据
	 */
	public byte[] encrypt(byte[] data, byte[] key) {
		try {
			SecureRandom sr = new SecureRandom();
			DESKeySpec dks = new DESKeySpec(key);
			SecretKeyFactory keyFactory = prv == null ? SecretKeyFactory.getInstance("DES") 
										: SecretKeyFactory.getInstance("DES", prv);
			SecretKey secretKey = keyFactory.generateSecret(dks);
			Cipher cipher = prv == null ? Cipher.getInstance("DES/ECB/PKCS5Padding")
										: Cipher.getInstance("DES/ECB/PKCS5Padding", prv);
			cipher.init(Cipher.ENCRYPT_MODE, secretKey, sr);
			byte encryptedData[] = cipher.doFinal(data);
			return encryptedData;
		} catch (Exception e) {
			log.error("DES算法，加密出错 ，" + e.getMessage());
			throw new RuntimeException("DES算法，加密数据出错!", e);
		}
	}
	
	/**
	 * 解密函数
	 * 
	 * @param data 解密数据
	 * @param key 密钥
	 * @return 返回解密后的数据
	 */
	public byte[] decrypt(byte[] data, byte[] key) {
		try {
			SecureRandom sr = new SecureRandom();
			DESKeySpec dks = new DESKeySpec(key);
			SecretKeyFactory keyFactory = prv == null ? SecretKeyFactory.getInstance("DES")
										: SecretKeyFactory.getInstance("DES", prv);
			SecretKey secretKey = keyFactory.generateSecret(dks);
			Cipher cipher = prv == null ? Cipher.getInstance("DES/ECB/PKCS5Padding")
										: Cipher.getInstance("DES/ECB/PKCS5Padding", prv);
			cipher.init(Cipher.DECRYPT_MODE, secretKey, sr);
			byte decryptedData[] = cipher.doFinal(data);
			return decryptedData;
		} catch (Exception e) {
			log.error("DES算法，解密出错 ，" + e.getMessage());
			throw new RuntimeException("DES算法，解密数据出错!", e);
		}

	}
	
	/**
	 * 二行制转16进制字符串
	 * 
	 * @param b
	 * @return
	 */
	public String byte2hex(byte[] b) {
		// 一个字节的数，转成16进制字符串
		String hs = "";
		String stmp = "";
		for (int n = 0; n < b.length; n++) {
			// 整数转成十六进制表示
			stmp = (java.lang.Integer.toHexString(b[n] & 0XFF));
			if (stmp.length() == 1)
				hs = hs + "0" + stmp;
			else
				hs = hs + stmp;
		}
		return hs.toUpperCase(); // 转成大写
	}

	/**
	 * 16进制字符串转二行制
	 * 
	 * @param b
	 * @return
	 */
	public byte[] hex2byte(byte[] b) {
		if ((b.length % 2) != 0)
			throw new IllegalArgumentException("长度不是偶数");
		byte[] b2 = new byte[b.length / 2];
		for (int n = 0; n < b.length; n += 2) {
			String item = new String(b, n, 2);
			// 两位一组，表示一个字节,把这样表示的16进制字符串，还原成一个进制字节
			b2[n / 2] = (byte) Integer.parseInt(item, 16);
		}

		return b2;
	}
}
