package com.allinpay.demo.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.security.MessageDigest;
import java.security.Provider;
import java.security.Security;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.bouncycastle.jce.provider.BouncyCastleProvider;

import sun.misc.BASE64Encoder;

import com.allinpay.demo.AIPGException;
import com.allinpay.demo.DemoConfig;
import com.allinpay.demo.xstruct.common.ReqHead;

/**
 * @Description
 * @Author meixf@allinpay.com
 * @Date 2018年5月24日
 **/
@SuppressWarnings("restriction")
public class DemoUtil {
	private static Provider prvd = null;
	
    static{
        System.out.println("初始化BC," + System.getProperty("addprovider"));
        prvd = new BouncyCastleProvider();  //密码不能用特殊符号
        if("true".equals(System.getProperty("addprovider")) && Security.getProvider("BC") == null){
            System.out.println("未找到BCProvider");
            Security.addProvider(prvd);
        }
    }
	
	public static ReqHead makeReq(String trxcod){
		ReqHead head=new ReqHead();
		head.setTRX_CODE(trxcod);
		head.setREQ_SN(DemoConfig.merchantid + String.format("-%016d",System.currentTimeMillis()));
		head.setUSER_NAME(DemoConfig.username);
		head.setVERSION("03");
		head.setLEVEL("5");
		head.setMERCHANT_ID(DemoConfig.merchantid);
		head.setSUBMIT_TIME(DemoUtil.getNow());
		return head;
	}
	
	public static String getNow(){
		SimpleDateFormat df = new SimpleDateFormat("yyyyMMddHHmmss");
		return df.format(new Date());
	}
	
	/**
	 * @param xmlMsg   待签名报文
	 * @param pathPfx   
	 * @param pass
	 * @return  签名后的报文
	 * @throws AIPGException
	 */
	public static String buildSignedXml(String xmlMsg) throws AIPGException{
		if(xmlMsg == null){
			throw new AIPGException("传入的加签报文为空");
		}
		AIPGSignature signature = new AIPGSignature();
		String signedStr;
		try {
			signedStr = signature.signMsg(xmlMsg, DemoConfig.pathpfx, DemoConfig.pfxpass);
		} catch (Exception e) {
			throw new AIPGException("加签失败", e);
		}
		return xmlMsg + "<SIGNED_MSG>" + signedStr + "</SIGNED_MSG>";
	}
	
	/**
	 * @param xmlMsg  返回报文
	 * @param pathCer  通联公钥
	 * @return
	 * @throws AIPGException
	 */
	public static boolean verifyXml(String xmlMsg) throws AIPGException{
		if(xmlMsg == null){
			throw new AIPGException("传入的验签报文为空");
		}
		int signPreIndex = xmlMsg.lastIndexOf("<SIGNED_MSG>");
		int signSufIndex = xmlMsg.lastIndexOf("</SIGNED_MSG>");
		String msgStr = xmlMsg.substring(0, signPreIndex);
		if(signPreIndex == - 1 || signSufIndex == -1 || signPreIndex >= signSufIndex){
			throw new AIPGException("找不到签名信息");
		}
		String signedStr = xmlMsg.substring(signPreIndex + 12, signSufIndex);
		System.out.println(signedStr.length());
		AIPGSignature signature = new AIPGSignature(prvd);
		try {
			return signature.verifyMsg(signedStr, msgStr, DemoConfig.pathcer);
		} catch (Exception e) {
			throw new AIPGException("验签失败", e);
		}
	}
	
	public static String getSha1Str(File file) {
		String str = "";
		try {
			MessageDigest md = MessageDigest.getInstance("SHA-1");
			FileInputStream fis = new FileInputStream(file);
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte[] buf = new byte[2048];
			int c;
			while ((c = fis.read(buf)) > 0) {
				baos.write(buf, 0, c);
			}
			str = new BASE64Encoder().encode(md.digest(baos.toByteArray()));
			fis.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return str;
	}
}
