package com.allinpay.demo.xml;

import com.allinpay.demo.AIPGException;
import com.allinpay.demo.xstruct.common.AipgReq;
import com.allinpay.demo.xstruct.common.AipgRsp;
import com.allinpay.demo.xstruct.common.ReqHead;
import com.allinpay.demo.xstruct.common.RspHead;
import com.allinpay.demo.xstruct.trans.Refund;
import com.allinpay.demo.xstruct.trans.TRANS;
import com.allinpay.demo.xstruct.trans.TRANSRET;
import com.allinpay.demo.xstruct.trans.authen.AGRINFO;
import com.allinpay.demo.xstruct.trans.authen.AGRRSP;
import com.allinpay.demo.xstruct.trans.authen.QAGRINFO;
import com.allinpay.demo.xstruct.trans.authen.QAGRRSP;
import com.allinpay.demo.xstruct.trans.breq.TRANS_DETAIL;
import com.allinpay.demo.xstruct.trans.breq.TRANS_DETAILS;
import com.allinpay.demo.xstruct.trans.breq.TRANS_SUM;
import com.allinpay.demo.xstruct.trans.other.DOWN_RSP;
import com.allinpay.demo.xstruct.trans.other.RECONF_DOWN;
import com.allinpay.demo.xstruct.trans.qry.ACCTDTL;
import com.allinpay.demo.xstruct.trans.qry.ACCTDTLQRY;
import com.allinpay.demo.xstruct.trans.qry.ACCTDTLRSP;
import com.allinpay.demo.xstruct.trans.qry.ACCTDTLS;
import com.allinpay.demo.xstruct.trans.qry.ACCTINFO;
import com.allinpay.demo.xstruct.trans.qry.ACCTQUERY;
import com.allinpay.demo.xstruct.trans.qry.QTDETAIL;
import com.allinpay.demo.xstruct.trans.qry.QTDETAILS;
import com.allinpay.demo.xstruct.trans.qry.QTRANSREQ;
import com.allinpay.demo.xstruct.trans.qry.QTSUM;
import com.thoughtworks.xstream.XStream;

/**
 * @Description
 * @Author meixf@allinpay.com
 * @Date 2018年5月23日
 **/
public class XmlParser {
	public static final String HEAD = "<?xml version=\"1.0\" encoding=\"GBK\"?>";
	private static final XStream reqxs = initXStream(new XStreamEx(), true);
	private static final XStream rspxs = initXStream(new XStream(), false);
	
	public static AipgReq toAipgReq(String xml) throws AIPGException{
		Object obj = reqxs.fromXML(xml);
		try {
			return AipgReq.class.cast(obj);
		} catch (Exception e) {
			throw new AIPGException("toAipgReq [" + xml +"], 出错：" + e.getMessage(), e);
		}
	}

	public static AipgRsp toAipgRsp(String xml) throws AIPGException{
		Object obj = rspxs.fromXML(xml);
		try {
			return AipgRsp.class.cast(obj);
		} catch (Exception e) {
			throw new AIPGException("toAipgRsp [" + xml +"], 出错：" + e.getMessage(), e);
		}
	}
	
	public static String toReqXml(Object o) {
		String xml = reqxs.toXML(o);
		xml = xml.replace("__", "_");
		xml = HEAD + xml;
		return xml;
	}

	public static String toRspXml(Object o){
		String xml = rspxs.toXML(o);
		xml = xml.replace("__", "_");
		xml = HEAD + xml;
		return xml;
	}
	
	public static XStream initXStream(XStream xs, boolean isreq) {
		if(isreq){
			xs.alias("AIPG", AipgReq.class);
			xs.alias("HEAD", ReqHead.class);
		}else{
			xs.alias("AIPG", AipgRsp.class);
			xs.alias("HEAD", RspHead.class);
		}
		xs.alias("TRANS", TRANS.class);
		xs.alias("TRANSRET", TRANSRET.class);
		xs.alias("TRANS_SUM", TRANS_SUM.class);
		xs.alias("TRANS_DETAILS", TRANS_DETAILS.class);
		xs.addImplicitCollection(TRANS_DETAILS.class, "details");
		xs.alias("TRANS_DETAIL", TRANS_DETAIL.class);
		
		xs.alias("QTDETAIL", QTDETAIL.class);
		xs.alias("QTRANSREQ", QTRANSREQ.class);
		xs.alias("QTDETAILS", QTDETAILS.class);
		xs.addImplicitCollection(QTDETAILS.class, "details");
		xs.alias("QTSUM", QTSUM.class);
		
		xs.alias("REFUND", Refund.class);
		
		xs.alias("DOWN_RSP", DOWN_RSP.class);
		xs.alias("RECONF_DOWN", RECONF_DOWN.class);
		
		xs.alias("ACCTQUERY", ACCTQUERY.class);
		xs.alias("ACCTINFO", ACCTINFO.class);
		
		xs.alias("ACCTDTL", ACCTDTL.class);
		xs.alias("ACCTDTLQRY", ACCTDTLQRY.class);
		xs.alias("ACCTDTLRSP", ACCTDTLRSP.class);
		xs.alias("ACCTDTLS", ACCTDTLS.class);
		xs.addImplicitCollection(ACCTDTLS.class, "details");
		
        xs.alias("AGRINFO", AGRINFO.class);
        xs.alias("AGRRSP", AGRRSP.class);
        xs.alias("QAGRINFO", QAGRINFO.class);
        xs.alias("QAGRRSP", QAGRRSP.class);
		return xs;
	}

}
